#include <WiFi.h>
#include <WiFiUdp.h>

// Soft AP credentials
const char* softApSsid = "Master";
const char* softApPassword = "12345678";

// Static IP config for Soft AP
IPAddress staticIP(192, 168, 4, 1);
IPAddress gateway(192, 168, 4, 1);
IPAddress subnet(255, 255, 255, 0);

// UDP parameters
const int localPort = 12345;
WiFiUDP udp;

char incomingPacket[255];

void setup() {
  Serial.begin(115200);

  // Setup Wi-Fi Soft AP with static IP
  WiFi.mode(WIFI_AP);
  WiFi.softAPConfig(staticIP, gateway, subnet);
  WiFi.softAP(softApSsid, softApPassword);

  Serial.println("ESP32 Soft AP started");
  Serial.print("AP IP address: ");
  Serial.println(WiFi.softAPIP());

  // Start UDP server
  udp.begin(localPort);
  Serial.println("UDP server started, waiting for packets...");
}

void loop() {
  int packetSize = udp.parsePacket();
  if (packetSize) {
    int len = udp.read(incomingPacket, 255);
    if (len > 0) {
      incomingPacket[len] = 0; // Null terminate string
    }

    // Example expected format: "0,12,34,56,78,90,123"
    // DeviceID, ax, ay, az, red, ir, heading
    Serial.print("Received raw: ");
    Serial.println(incomingPacket);

    // Parse CSV values
    int values[7] = {0};  // Adjust size based on your data count
    int valIndex = 0;

    char *token = strtok(incomingPacket, ",");
    while (token != NULL && valIndex < 7) {
      values[valIndex] = atoi(token);
      valIndex++;
      token = strtok(NULL, ",");
    }

    if (valIndex >= 1) {
      int deviceId = values[0];
      Serial.print("Device ");
      Serial.print(deviceId);
      Serial.println(" data:");

      if (valIndex >= 4) {
        Serial.print("  Accel X: "); Serial.println(values[1]);
        Serial.print("  Accel Y: "); Serial.println(values[2]);
        Serial.print("  Accel Z: "); Serial.println(values[3]);
      }
      if (valIndex >= 6) {
        Serial.print("  MAX30102 Red: "); Serial.println(values[4]);
        Serial.print("  MAX30102 IR: "); Serial.println(values[5]);
      }
      if (valIndex == 7) {
        Serial.print("  Magnetometer Heading: "); Serial.println(values[6]);
      }
      Serial.println("--------------------");
    } else {
      Serial.println("Invalid data format");
    }
  }

  delay(10);
}
